import 'package:equatable/equatable.dart';
import 'package:extensions_pack/extensions_pack.dart';
import 'package:flutter/material.dart';
import 'package:widgets_pack/widgets/table/table.dart';

sealed class TableColumn<M extends Object> extends Equatable {
  final Widget label;
  final bool numeric;
  final double width;
  final bool fixed;

  const TableColumn({
    required this.label,
    this.numeric = false,
    this.width = 350,
    this.fixed = false,
  });

  @override
  List<Object?> get props => [label, numeric, width, fixed];

  Widget _builder(
    M model,
    TableController<M> controller,
  ) {
    throw UnimplementedError();
  }

  Widget labelBuilder() {
    return Container(
      width: width,
      padding: const EdgeInsets.symmetric(horizontal: kXSSize),
      child: label,
    );
  }

  Widget contentBuilder(M model, TableController<M> controller) {
    return Container(
      width: width,
      padding: const EdgeInsets.symmetric(horizontal: kXSSize),
      child: Align(
        alignment: numeric ? Alignment.centerRight : Alignment.centerLeft,
        child: _builder(model, controller),
      ),
    );
  }
}

class TextColumn<M extends Object> extends TableColumn<M> {
  final String? Function(M model) value;

  const TextColumn({
    required super.label,
    required this.value,
    super.numeric,
    super.width,
    super.fixed,
  });

  @override
  Widget _builder(
    M model,
    TableController<M> controller,
  ) {
    return Text(value(model) ?? '-');
  }
}

class WidgetColumn<M extends Object> extends TableColumn<M> {
  final Widget Function(M, TableController<M>) builder;

  const WidgetColumn({
    required super.label,
    required this.builder,
    super.numeric,
    super.width,
    super.fixed,
  });

  @override
  Widget _builder(
    M model,
    TableController<M> controller,
  ) {
    return builder(model, controller);
  }
}
